import { HttpClient } from '@angular/common/http';
import { Component, ElementRef, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { CellEditConfig, CellMenuConfig, EventDragCreateConfig, EventMenuConfig, ProjectModel, ResourceNonWorkingTimeConfig, ScheduleMenuConfig, SchedulerPro, TaskEditConfig, TimeRangesConfig } from '@bryntum/gantt';
import { zip } from 'rxjs';


@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
  encapsulation: ViewEncapsulation.None
})
export class AppComponent implements OnInit {
  @ViewChild('schedulerDiv', { static: true }) private schedulerDiv!: ElementRef;

  private projectStart = new Date(2021, 8, 5);
  private projectEnd = new Date(2021, 8, 15);

  private schedulerInstance!: SchedulerPro;

  constructor(
    private readonly http: HttpClient
  ) {}

  ngOnInit(): void {
    this.renderGantt();
    this.loadProject();
  }

  private renderGantt(): void {
    const schedulerInstance = new SchedulerPro({
      appendTo: this.schedulerDiv.nativeElement,
      columns: [
        {
          type: 'resourceInfo',
          field: 'name',
          text:'Resource',
          minWidth: 200,
          showEventCount: true,
          showImage: false
        }
      ],
      features: {
        percentBar: false,
        taskEdit: <TaskEditConfig> {
          disabled: false,
        },
        resourceNonWorkingTime: <ResourceNonWorkingTimeConfig> {
          maxTimeAxisUnit: 'year',
          disabled: false
        },
        eventDragCreate: <EventDragCreateConfig> {
          disabled: true
        },
        timeRanges: <TimeRangesConfig> {
          showCurrentTimeLine: {
            name: 'Now'
          },
          disabled: false,
          showHeaderElements: true
        },
        eventMenu: <EventMenuConfig> {
          disabled: false,
          items: {
            deleteEvent: false,
            unassignEvent: false,
            extraItem: {
              text: 'My bytton on PPM',
              onItem: (eventInfo: any) => {
                // code
              }
            }
          },
        },
        scheduleMenu: <ScheduleMenuConfig> {
          disabled: false,
          items: {
            addEvent: false
          }
        },
        cellMenu: <CellMenuConfig> {
          disabled: true
        },
        cellEdit: <CellEditConfig> {
          disabled: true
        },
        dependencies: {
          allowCreation: false
        },
      },
      createEventOnDblClick: false,
      zoomKeepsOriginalTimespan: true,
      rowHeight: 60,
      readOnly: false,
    });

    this.schedulerInstance = schedulerInstance;
  }

  private loadProject(): void {
    setTimeout(() => {
      zip(
        this.http.get('../assets/json/resources.json'),
        this.http.get('../assets/json/assignments.json'),
        this.http.get('../assets/json/tasks.json')
      ).subscribe(
        (values: any[]) => this.onLoadProject(values),
        (err: any) => console.error(err));
    }, 2500);
  }

  private async onLoadProject(values: any[]): Promise<void> {
    // 0 resources
    // 1 assignments
    // 2 tasks

    const project = new ProjectModel({
      resourcesData: <any>values[0],
      assignmentsData: <any>values[1],
      tasksData: <any>values[2],
    });



    this.schedulerInstance.project = <any>project;

    this.setSchedulerTimelineRanges(this.projectStart, this.projectEnd);
    this.loadWorkingTimes();
  }

  private setSchedulerTimelineRanges(startDate: Date, endDate: Date): void {
    this.schedulerInstance.timeRanges.push(<any> {
      id: 'projectstart',
      name:'Start',
      startDate: startDate
    });

    this.schedulerInstance.timeRanges.push(<any> {
      id: 'projectend',
      name: 'End',
      startDate: endDate
    });

    this.schedulerInstance.setEndDate(endDate);
    this.schedulerInstance.setStartDate(startDate);
    this.schedulerInstance.setTimeSpan(startDate, endDate);

    this.schedulerInstance.onPresetChange = () => {
      this.schedulerInstance.setTimeSpan(startDate, endDate);
    };

    this.schedulerInstance.scrollToDate(new Date(), {
      block: 'center',
      animate: 800
    });
  }

  private loadWorkingTimes(): void {
    setTimeout(() => {
      this.http.get('../assets/json/resources-working-time.json').subscribe(
        (resourcesWorkingTime: any) => this.onLoadWorkingTimes(resourcesWorkingTime)
      );
    }, 5000);
  }

  private onLoadWorkingTimes(resourcesWorkingTime: any[]): void {
    (<ProjectModel><any>this.schedulerInstance.project).calendarManagerStore.removeAll();

    for(const resourceWorkingTime of resourcesWorkingTime) {
      const ganttResource  = this.schedulerInstance.resources.find((x: any) => x.id === resourceWorkingTime.resourceId);

      const workingIntervals: any[] = resourceWorkingTime.ranges.map((x: any) => {
        return {
          isWorking: true,
          endDate: new Date(x.endDate),
          startDate: new Date(x.startDate)
        }
      });

      const calendarId: string = 'calendar_' + (<any>ganttResource).id;

      (<ProjectModel><any>this.schedulerInstance.project).calendarManagerStore.add({
        id: calendarId,
        name: (<any>ganttResource).name,
        unspecifiedTimeIsWorking: false,
        intervals: workingIntervals
      });

      (<any>ganttResource).calendar = calendarId;
    }
  }
}
